"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const rimraf = require("rimraf");
const config_1 = require("../../../../../config");
const profiles_1 = require("../../../../../profiles");
const utilities_1 = require("../../../../../utilities");
const settings_1 = require("../../../../../settings");
const PluginIssues_1 = require("../../../plugins/utilities/PluginIssues");
const npm_interface_1 = require("../../../plugins/utilities/npm-interface");
const OverridesLoader_1 = require("../../../OverridesLoader");
/**
 * Handler for the convert profiles command.
 */
class ConvertProfilesHandler {
    constructor() {
        this.ZOWE_CLI_PACKAGE_NAME = "@zowe/cli";
        this.ZOWE_CLI_SECURE_PLUGIN_NAME = "@zowe/secure-credential-store-for-zowe-cli";
        this.keytar = undefined;
    }
    /**
     * Process the command and input.
     *
     * @param {IHandlerParameters} params Parameters supplied by yargs
     *
     * @throws {ImperativeError}
     */
    process(params) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const cliBin = utilities_1.ImperativeConfig.instance.rootCommandName;
            const profilesRootDir = profiles_1.ProfileUtils.constructProfilesRootDirectory(utilities_1.ImperativeConfig.instance.cliHome);
            const configExists = (_a = utilities_1.ImperativeConfig.instance.config) === null || _a === void 0 ? void 0 : _a.exists;
            const oldPluginInfo = this.getOldPluginInfo();
            // Cannot do profiles operations w/ team config
            const oldProfileCount = configExists ? 0 : this.getOldProfileCount(profilesRootDir);
            const oldProfilesDir = `${profilesRootDir.replace(/[\\/]$/, "")}-old`;
            let skipConversion = false;
            if (configExists) {
                // Warn that a team config was detected
                params.response.console.log(`A team configuration file was detected. V1 profiles cannot be loaded for conversion.\n` +
                    `Run '${cliBin} config list --locations --root' for team configuration file locations.\n`);
            }
            if (oldPluginInfo.plugins.length == 0 && oldProfileCount === 0) {
                params.response.console.log("No old profiles or plug-ins were found to convert from Zowe v1 to v2.");
                // Exit if we're not deleting
                if (!(params.arguments.delete != null && params.arguments.delete === true)) {
                    return;
                }
                else {
                    skipConversion = true;
                }
            }
            // If this is true, then we know that we want to delete, but there is nothing to convert first.
            if (!skipConversion) {
                const listToConvert = [];
                if (oldPluginInfo.plugins.length > 0) {
                    listToConvert.push(`${oldPluginInfo.plugins.length} obsolete plug-in(s)`);
                }
                if (oldProfileCount > 0) {
                    listToConvert.push(`${oldProfileCount} old profile(s)`);
                }
                params.response.console.log(`Detected ${listToConvert.join(" and ")} to convert from Zowe v1 to v2.\n`);
                if (params.arguments.prompt == null || params.arguments.prompt === true) {
                    const answer = yield params.response.console.prompt("Are you sure you want to continue? [y/N]: ");
                    if (answer == null || !(answer.toLowerCase() === "y" || answer.toLowerCase() === "yes")) {
                        return;
                    }
                }
                params.response.console.log("");
                oldPluginInfo.overrides.forEach(this.removeOverride);
                for (const pluginName of oldPluginInfo.plugins) {
                    try {
                        npm_interface_1.uninstall(pluginName);
                        params.response.console.log(`Removed obsolete plug-in: ${pluginName}`);
                    }
                    catch (error) {
                        params.response.console.error(`Failed to uninstall plug-in "${pluginName}":\n    ${error}`);
                    }
                }
                if (oldProfileCount != 0) {
                    yield OverridesLoader_1.OverridesLoader.ensureCredentialManagerLoaded();
                    const convertResult = yield config_1.ConfigBuilder.convert(profilesRootDir);
                    for (const [k, v] of Object.entries(convertResult.profilesConverted)) {
                        params.response.console.log(`Converted ${k} profiles: ${v.join(", ")}`);
                    }
                    if (convertResult.profilesFailed.length > 0) {
                        params.response.console.log("");
                        params.response.console.errorHeader(`Failed to convert ${convertResult.profilesFailed.length} profile(s). See details below`);
                        for (const { name, type, error } of convertResult.profilesFailed) {
                            if (name != null) {
                                params.response.console.error(`Failed to load ${type} profile "${name}":\n    ${error}`);
                            }
                            else {
                                params.response.console.error(`Failed to find default ${type} profile:\n    ${error}`);
                            }
                        }
                    }
                    params.response.console.log("");
                    const teamConfig = utilities_1.ImperativeConfig.instance.config;
                    teamConfig.api.layers.activate(false, true);
                    teamConfig.api.layers.merge(convertResult.config);
                    config_1.ConfigSchema.updateSchema();
                    yield teamConfig.save(false);
                    try {
                        fs.renameSync(profilesRootDir, oldProfilesDir);
                    }
                    catch (error) {
                        params.response.console.error(`Failed to rename profiles directory to ${oldProfilesDir}:\n    ${error}`);
                    }
                    params.response.console.log(`Your new profiles have been saved to ${teamConfig.layerActive().path}.\n` +
                        `Run "${cliBin} config edit --global-config" to open this file in your default editor.\n`);
                    if (params.arguments.delete == null || params.arguments.delete === false) {
                        params.response.console.log(`Your old profiles have been moved to ${oldProfilesDir}.\n` +
                            `Run "${cliBin} config convert-profiles --delete" if you want to completely remove them.\n\n` +
                            `If you would like to revert back to v1 profiles, or convert your v1 profiles again, rename the 'profiles-old'\n` +
                            `directory to 'profiles' and delete the new config file located at ${teamConfig.layerActive().path}.`);
                    }
                }
            }
            if (params.arguments.delete != null && params.arguments.delete === true) {
                if (params.arguments.prompt == null || params.arguments.prompt === true) {
                    const answer = yield params.response.console.prompt("Are you sure you want to delete your v1 profiles? [y/N]: ");
                    if (answer == null || !(answer.toLowerCase() === "y" || answer.toLowerCase() === "yes")) {
                        return;
                    }
                }
                // Delete the profiles directory
                try {
                    rimraf.sync(oldProfilesDir);
                    params.response.console.log(`Deleting the profiles directory '${oldProfilesDir}'... done`);
                }
                catch (err) {
                    params.response.console.error(`Failed to delete the profiles directory '${oldProfilesDir}':\n    ${err}`);
                }
                // Delete the securely stored credentials
                const keytarAvailable = yield this.checkKeytarAvailable();
                if (keytarAvailable) {
                    const knownServices = ["@brightside/core", "@zowe/cli", "Zowe-Plugin", "Broadcom-Plugin", "Zowe"];
                    for (const service of knownServices) {
                        const accounts = yield this.findOldSecureProps(service, params);
                        for (const account of accounts) {
                            if (!account.includes("secure_config_props")) {
                                const success = this.deleteOldSecureProps(service, account, params);
                                params.response.console.log(`Deleting secure value for "${service}/${account}"... ${success ? "done" : "failed"}`);
                            }
                        }
                    }
                }
                else {
                    params.response.console.error(`Keytar or the credential vault are unavailable. Unable to delete old secure values.`);
                }
            }
        });
    }
    /**
     * Retrieve info about old plug-ins and their overrides.
     *  - `plugins` - List of CLI plug-ins to uninstall
     *  - `overrides` - List of overrides to remove from app settings
     */
    getOldPluginInfo() {
        const pluginInfo = {
            plugins: [],
            overrides: []
        };
        if (utilities_1.ImperativeConfig.instance.hostPackageName === this.ZOWE_CLI_PACKAGE_NAME) {
            let oldCredMgr = settings_1.AppSettings.instance.get("overrides", "CredentialManager");
            if (typeof oldCredMgr !== "string" || oldCredMgr === utilities_1.ImperativeConfig.instance.hostPackageName) {
                // Fall back to default plug-in name because CredentialManager override is not set
                oldCredMgr = this.ZOWE_CLI_SECURE_PLUGIN_NAME;
            }
            else {
                // Need to remove CredentialManager override because it is a plug-in name
                pluginInfo.overrides.push("CredentialManager");
            }
            // Only uninstall plug-in if it is currently installed
            if (oldCredMgr in PluginIssues_1.PluginIssues.instance.getInstalledPlugins()) {
                pluginInfo.plugins.push(oldCredMgr);
            }
        }
        return pluginInfo;
    }
    /**
     * Get the number of old profiles present in the CLI home dir.
     * @param profilesRootDir Root profiles directory
     * @returns Number of old profiles found
     */
    getOldProfileCount(profilesRootDir) {
        const profileTypes = profiles_1.ProfileIO.getAllProfileDirectories(profilesRootDir);
        let oldProfileCount = 0;
        for (const profileType of profileTypes) {
            const profileTypeDir = path.join(profilesRootDir, profileType);
            const profileNames = profiles_1.ProfileIO.getAllProfileNames(profileTypeDir, ".yaml", `${profileType}_meta`);
            oldProfileCount += profileNames.length;
        }
        return oldProfileCount;
    }
    /**
     * Remove obsolete Imperative overrides from app settings. This method is
     * called before uninstalling old plug-ins.
     *
     * This method is private because only the convert-profiles command is able
     * to disable the credential manager and reload it. For all other commands,
     * the credential manager is loaded in `Imperative.init` and frozen with
     * `Object.freeze` so cannot be modified later on.
     */
    removeOverride(override) {
        switch (override) {
            case "CredentialManager":
                settings_1.AppSettings.instance.set("overrides", "CredentialManager", utilities_1.ImperativeConfig.instance.hostPackageName);
                if (utilities_1.ImperativeConfig.instance.loadedConfig.overrides.CredentialManager != null) {
                    delete utilities_1.ImperativeConfig.instance.loadedConfig.overrides.CredentialManager;
                }
                break;
        }
    }
    /**
     * Lazy load keytar, and verify that the credential vault is able to be accessed,
     * or whether there is a problem.
     * @returns true if credential vault is available, false if it is not
     */
    checkKeytarAvailable() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            let success = false;
            const requireOpts = {};
            if (((_a = process.mainModule) === null || _a === void 0 ? void 0 : _a.filename) != null) {
                requireOpts.paths = [process.mainModule.filename];
            }
            try {
                const keytarPath = require.resolve("keytar", requireOpts);
                this.keytar = yield Promise.resolve().then(() => require(keytarPath));
                yield this.keytar.findCredentials(this.ZOWE_CLI_PACKAGE_NAME);
                success = true;
            }
            catch (err) {
                success = false;
            }
            return success;
        });
    }
    /**
     * Locate the names of secured properties stored under an account in the operating
     * system's credential vault.
     * @param acct The account to search for in the credential store
     * @param params The parameters and response console APIs
     * @returns a list of secured properties stored under the specified account
     */
    findOldSecureProps(acct, params) {
        return __awaiter(this, void 0, void 0, function* () {
            const oldSecurePropNames = [];
            try {
                const credentialsArray = yield this.keytar.findCredentials(acct);
                for (const element of credentialsArray) {
                    oldSecurePropNames.push(element.account);
                }
            }
            catch (err) {
                params.response.console.error(`Encountered an error while gathering profiles for service '${acct}':\n    ${err}`);
            }
            return oldSecurePropNames;
        });
    }
    /**
     * Delete the secure property specified from the operating system credential vault.
     * @param acct The account the property is stored under
     * @param propName The name of the property to delete
     * @param params The parameters and response console APIs
     * @returns true if the property was deleted successfully
     */
    deleteOldSecureProps(acct, propName, params) {
        return __awaiter(this, void 0, void 0, function* () {
            let success = false;
            try {
                success = yield this.keytar.deletePassword(acct, propName);
            }
            catch (err) {
                params.response.console.error(`Encountered an error while deleting secure data for service '${acct}/${propName}':\n    ${err}`);
                success = false;
            }
            return success;
        });
    }
}
exports.default = ConvertProfilesHandler;
//# sourceMappingURL=convert-profiles.handler.js.map