"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const url_1 = require("url");
const JSONC = require("comment-json");
const utilities_1 = require("../../../../../utilities");
const rest_1 = require("../../../../../rest");
/**
 * Import config
 */
class ImportHandler {
    /**
     * Process the command and input.
     *
     * @param {IHandlerParameters} params Parameters supplied by yargs
     *
     * @throws {ImperativeError}
     */
    process(params) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            // Load the config and set the active layer according to user options
            const config = utilities_1.ImperativeConfig.instance.config;
            const configDir = params.arguments.globalConfig ? null : process.cwd();
            config.api.layers.activate(params.arguments.userConfig, params.arguments.globalConfig, configDir);
            const layer = config.api.layers.get();
            if (layer.exists && !params.arguments.overwrite) {
                params.response.console.log(`Skipping import because ${layer.path} already exists.\n` +
                    `Rerun the command with the --overwrite flag to import anyway.`);
                return;
            }
            const configFilePath = path.resolve(params.arguments.location);
            const isConfigLocal = fs.existsSync(configFilePath) || path.isAbsolute(params.arguments.location);
            const configJson = isConfigLocal ?
                JSONC.parse(fs.readFileSync(configFilePath, "utf-8")) :
                yield this.fetchConfig(new url_1.URL(params.arguments.location));
            config.api.layers.set(configJson);
            let schemaImported = false;
            if ((_a = configJson.$schema) === null || _a === void 0 ? void 0 : _a.startsWith("./")) { // Only import schema if relative path
                const schemaUri = new url_1.URL(configJson.$schema, isConfigLocal ? url_1.pathToFileURL(configFilePath) : params.arguments.location);
                const schemaFilePath = path.resolve(path.dirname(layer.path), configJson.$schema);
                try {
                    yield this.downloadSchema(schemaUri, schemaFilePath);
                    schemaImported = true;
                }
                catch (error) {
                    params.response.console.error(`Failed to download schema from ${schemaUri}`);
                }
            }
            // Write the active created/updated config layer
            yield config.api.layers.write();
            params.response.console.log(`Imported config${schemaImported ? " and schema" : ""} to ${layer.path}`);
        });
    }
    /**
     * Download the config from a URL
     * @param url Web URL of the config JSON file
     * @returns Parsed config object
     */
    fetchConfig(url) {
        return __awaiter(this, void 0, void 0, function* () {
            const session = rest_1.Session.createFromUrl(url, false);
            const response = yield rest_1.RestClient.getExpectString(session, url.pathname);
            return JSONC.parse(response);
        });
    }
    /**
     * Download the config schema from a URL to disk
     * @param url Web URL of the schema JSON file
     * @param filePath Local path to download to
     */
    downloadSchema(url, filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (url.protocol === "file:") {
                fs.copyFileSync(url_1.fileURLToPath(url), filePath);
            }
            else {
                const session = rest_1.Session.createFromUrl(url, false);
                const response = yield rest_1.RestClient.getExpectString(session, url.pathname);
                fs.writeFileSync(filePath, response);
            }
        });
    }
}
exports.default = ImportHandler;
//# sourceMappingURL=import.handler.js.map