"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("../../../../../config");
const ConfigUtils_1 = require("../../../../../config/src/ConfigUtils");
const error_1 = require("../../../../../error");
const logger_1 = require("../../../../../logger");
const rest_1 = require("../../../../../rest");
const utilities_1 = require("../../../../../utilities");
class SecureHandler {
    /**
     * Process the command and input.
     *
     * @param {IHandlerParameters} params Parameters supplied by yargs
     *
     * @throws {ImperativeError}
     */
    process(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this.params = params;
            const config = utilities_1.ImperativeConfig.instance.config;
            // Setup the credential vault API for the config
            if (config.api.secure.loadFailed) {
                throw ConfigUtils_1.secureSaveError();
            }
            // Create the config, load the secure values, and activate the desired layer
            config.api.layers.activate(params.arguments.userConfig, params.arguments.globalConfig);
            const secureProps = config.api.secure.secureFields();
            if (secureProps.length === 0) {
                params.response.console.log("No secure properties found in your config");
                return;
            }
            // Prompt for values designated as secure
            let authTokenProp;
            for (const propName of secureProps) {
                if (authTokenProp == null && propName.endsWith(".tokenValue")) {
                    authTokenProp = propName;
                    continue;
                }
                const propValue = yield params.response.console.prompt(`Please enter ${propName}: `, { hideText: true });
                // Save the value in the config securely
                if (propValue) {
                    config.set(propName, propValue, { parseString: true, secure: true });
                }
            }
            if (authTokenProp != null) {
                const propValue = (yield this.handlePromptForAuthToken(config, authTokenProp)) ||
                    (yield params.response.console.prompt(`Please enter ${authTokenProp}: `, { hideText: true }));
                // Save the value in the config securely
                if (propValue) {
                    config.set(authTokenProp, propValue, { secure: true });
                }
            }
            // Write the config layer
            yield config.save(false);
        });
    }
    /**
     * Checks if authentication service is associated with an auth token
     * property. If an auth service is found, we log in to it obtain the token
     * instead of prompting for the value.
     * @param config Team config class from which the property was loaded
     * @param propPath JSON property path of the auth token
     * @returns Token value, or undefined if none was obtained
     */
    handlePromptForAuthToken(config, propPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const profilePath = propPath.slice(0, propPath.indexOf(".properties"));
            const authHandlerClass = config_1.ConfigAutoStore.findAuthHandlerForProfile(profilePath, this.params.arguments);
            if (authHandlerClass != null) {
                const [promptParams, loginHandler] = authHandlerClass.getPromptParams();
                if (promptParams.serviceDescription != null) {
                    this.params.response.console.log(`Logging in to ${promptParams.serviceDescription}`);
                }
                const profile = config.api.profiles.get(profilePath.replace(/profiles\./g, ""));
                const sessCfg = yield rest_1.ConnectionPropsForSessCfg.addPropsOrPrompt({}, profile, Object.assign({ parms: this.params, doPrompting: true, requestToken: true }, promptParams));
                logger_1.Logger.getAppLogger().info(`Fetching ${profile.tokenType} for ${propPath}`);
                try {
                    return yield loginHandler(new rest_1.Session(sessCfg));
                }
                catch (error) {
                    throw new error_1.ImperativeError({
                        msg: `Failed to fetch ${profile.tokenType} for ${propPath}`,
                        causeErrors: error
                    });
                }
            }
        });
    }
}
exports.default = SecureHandler;
//# sourceMappingURL=secure.handler.js.map